@@ -1,8 +1,9 @@
-from expansion_nodes import QuadraticExpansionNode, PolynomialExpansionNode
 from pca_nodes import WhiteningNode, PCANode
 from sfa_nodes import SFANode, SFA2Node
 from ica_nodes import ICANode, CuBICANode, FastICANode, TDSEPNode
 from neural_gas_nodes import GrowingNeuralGasNode
+from expansion_nodes import (QuadraticExpansionNode, PolynomialExpansionNode,
+                             RBFExpansionNode, GrowingNeuralGasExpansionNode)
 from fda_nodes import FDANode
 from em_nodes import FANode
 from misc_nodes import (IdentityNode, HitParadeNode, TimeFramesNode,
@@ -12,6 +13,9 @@ from misc_nodes import (IdentityNode, HitParadeNode, TimeFramesNode,
 from isfa_nodes import ISFANode
 from rbm_nodes import RBMNode, RBMWithLabelsNode
 from regression_nodes import LinearRegressionNode
+from classifier_nodes import (SignumClassifier, PerceptronClassifier,
+                              SimpleMarkovClassifier, DiscreteHopfieldClassifier,
+                              KMeansClassifier)
 
 # import internals for use in test_suites
 from misc_nodes import OneDimensionalHitParade as _OneDimensionalHitParade
@@ -19,7 +23,7 @@ from expansion_nodes import expanded_dim as _expanded_dim
 
 # import contributed nodes
 import mdp
-from mdp.contrib import JADENode, NIPALSNode, LLENode, HLLENode, XSFANode
+from mdp.contrib import *
 
 # clean up namespace
 del expansion_nodes
@@ -33,16 +37,21 @@ del misc_nodes
 del isfa_nodes
 del rbm_nodes
 del regression_nodes
-del mdp.contrib
+del classifier_nodes
+#del mdp.contrib
 del mdp
 
 __all__ = ['CuBICANode', 'EtaComputerNode', 'FANode', 'FDANode', 'FastICANode',
            'GaussianClassifierNode', 'GrowingNeuralGasNode', 'HitParadeNode',
-           'ICANode', 'ISFANode', 'JADENode', 'LLENode', 'NIPALSNode',
+           'ICANode', 'ISFANode', 'JADENode', 'LLENode', 'LibSVMClassifier', 'NIPALSNode',
            'NoiseNode', 'NormalNoiseNode', 'HLLENode', 'IdentityNode',
            'PCANode', 'PolynomialExpansionNode', 'QuadraticExpansionNode',
-           'RBMNode', 'RBMWithLabelsNode', 'SFA2Node', 'SFANode',
+           'RBFExpansionNode',
+           'RBMNode', 'RBMWithLabelsNode', 'SFA2Node', 'SFANode', 'ShogunSVMClassifier'
            'TDSEPNode', 'TimeFramesNode','WhiteningNode', 'XSFANode',
            'LinearRegressionNode',
            '_OneDimensionalHitParade', '_expanded_dim',
-           'CutoffNode', 'HistogramNode', 'AdaptiveCutoffNode']
+           'CutoffNode', 'HistogramNode', 'AdaptiveCutoffNode',
+           'SignumClassifier', 'PerceptronClassifier',
+           'SimpleMarkovClassifier', 'DiscreteHopfieldClassifier',
+           'KMeansClassifier']
